'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.Emitter = exports.Result = exports.Event = exports.PRIORITY = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _freeze = require('babel-runtime/core-js/object/freeze');

var _freeze2 = _interopRequireDefault(_freeze);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PRIORITY = exports.PRIORITY = (0, _freeze2.default)({
    'HIGHEST': -1000,
    'HIGHER': -100,
    'HIGH': -10,
    'NORMAL': 0,
    'LOW': +10,
    'LOWER': +100,
    'LOWEST': +1000
});

/* Sequential ordering for events */
var ORDER = 0;

var Event = exports.Event = function () {
    function Event() {
        (0, _classCallCheck3.default)(this, Event);

        this._stopped = false;
        this._active = false;
        this._value = null;
    }

    (0, _createClass3.default)(Event, [{
        key: 'stop',
        value: function stop() {
            if (!this._active) {
                throw new Error('can\'t stop an event after it has ended!');
            }

            this._stopped = true;
        }
    }, {
        key: 'stopped',
        get: function get() {
            return this._stopped;
        }
    }, {
        key: 'value',
        get: function get() {
            return this._value;
        }
    }]);
    return Event;
}();

var Result = exports.Result = function () {
    function Result() {
        (0, _classCallCheck3.default)(this, Result);

        this.events = [];
    }

    (0, _createClass3.default)(Result, [{
        key: '_addEvent',
        value: function _addEvent($evt) {
            this.events.push($evt);
        }
    }, {
        key: 'stopped',
        get: function get() {
            return this.events.some(function ($evt) {
                return $evt.stopped;
            });
        }
    }, {
        key: 'values',
        get: function get() {
            var values = [];

            for (var evt in this.events) {
                var value = evt.value;

                if (value instanceof Result) {
                    values.push.apply(values, (0, _toConsumableArray3.default)(value.values));
                } else {
                    values.push(value);
                }
            }

            return values;
        }
    }]);
    return Result;
}();

var Emitter = exports.Emitter = function () {
    function Emitter(source) {
        (0, _classCallCheck3.default)(this, Emitter);

        this.source = source;
        this.listeners = new _map2.default();
    }

    (0, _createClass3.default)(Emitter, [{
        key: 'on',
        value: function on(name, hook) {
            var priority = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : PRIORITY.NORMAL;
            var thisArg = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : this;

            if (!this.listeners.has(name)) {
                this.listeners.set(name, []);
            }

            var funcs = this.listeners.get(name);

            funcs.push({
                'hook': hook,
                'this': thisArg,
                'order': ORDER++,
                'priority': priority
            });

            funcs.sort(function (a, b) {
                return a.priority - b.priority || a.order - b.order;
            });

            return hook;
        }
    }, {
        key: 'off',
        value: function off(name, hook) {
            // branch on arguments
            if (name && hook) return this._off_nameHook(name, hook);else if (name) return this._off_nameAll();else if (hook) return this._off_hookAny();else return this._off_all();
        }
    }, {
        key: '_off_all',
        value: function _off_all() {
            this.listeners.clear();
        }
    }, {
        key: '_off_nameAll',
        value: function _off_nameAll(name) {
            this.listeners.delete(name);
        }
    }, {
        key: '_off_hookAny',
        value: function _off_hookAny(hook) {
            var _this = this;

            this.listeners.forEach(function (_, name) {
                return _this._off_nameHook(name, hook);
            });
        }
    }, {
        key: '_off_nameHook',
        value: function _off_nameHook(name, hook) {
            var listeners = this.listeners.get(name) || [];
            for (var i in listeners) {
                var listener = listeners[i];

                if (hook === listener.hook) {
                    listeners.splice(i, 1);
                }
            }
        }
    }, {
        key: 'emit',
        value: function () {
            var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(names_) {
                for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                    args[_key - 1] = arguments[_key];
                }

                var names, $res, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, name, listeners, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2, _listener$hook, listener, $evt, res;

                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                names = Array.isArray(names_) ? names_ : [names_];
                                $res = new Result();
                                _iteratorNormalCompletion = true;
                                _didIteratorError = false;
                                _iteratorError = undefined;
                                _context.prev = 5;
                                _iterator = (0, _getIterator3.default)(names);

                            case 7:
                                if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                                    _context.next = 49;
                                    break;
                                }

                                name = _step.value;
                                listeners = this.listeners.get(name) || [];
                                _context.next = 12;
                                return _bluebird2.default.delay();

                            case 12:
                                // yield the current event for fully async emits

                                _iteratorNormalCompletion2 = true;
                                _didIteratorError2 = false;
                                _iteratorError2 = undefined;
                                _context.prev = 15;
                                _iterator2 = (0, _getIterator3.default)(listeners);

                            case 17:
                                if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
                                    _context.next = 32;
                                    break;
                                }

                                listener = _step2.value;
                                $evt = new Event();


                                $evt._active = true;
                                _context.next = 23;
                                return _bluebird2.default.cast((_listener$hook = listener.hook).call.apply(_listener$hook, [listener.thisArg, $evt].concat(args)));

                            case 23:
                                res = _context.sent;

                                $evt._active = false;

                                $evt._value = res;
                                $res._addEvent($evt);

                                if (!$evt.stopped) {
                                    _context.next = 29;
                                    break;
                                }

                                return _context.abrupt('break', 32);

                            case 29:
                                _iteratorNormalCompletion2 = true;
                                _context.next = 17;
                                break;

                            case 32:
                                _context.next = 38;
                                break;

                            case 34:
                                _context.prev = 34;
                                _context.t0 = _context['catch'](15);
                                _didIteratorError2 = true;
                                _iteratorError2 = _context.t0;

                            case 38:
                                _context.prev = 38;
                                _context.prev = 39;

                                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                                    _iterator2.return();
                                }

                            case 41:
                                _context.prev = 41;

                                if (!_didIteratorError2) {
                                    _context.next = 44;
                                    break;
                                }

                                throw _iteratorError2;

                            case 44:
                                return _context.finish(41);

                            case 45:
                                return _context.finish(38);

                            case 46:
                                _iteratorNormalCompletion = true;
                                _context.next = 7;
                                break;

                            case 49:
                                _context.next = 55;
                                break;

                            case 51:
                                _context.prev = 51;
                                _context.t1 = _context['catch'](5);
                                _didIteratorError = true;
                                _iteratorError = _context.t1;

                            case 55:
                                _context.prev = 55;
                                _context.prev = 56;

                                if (!_iteratorNormalCompletion && _iterator.return) {
                                    _iterator.return();
                                }

                            case 58:
                                _context.prev = 58;

                                if (!_didIteratorError) {
                                    _context.next = 61;
                                    break;
                                }

                                throw _iteratorError;

                            case 61:
                                return _context.finish(58);

                            case 62:
                                return _context.finish(55);

                            case 63:
                                return _context.abrupt('return', $res);

                            case 64:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, this, [[5, 51, 55, 63], [15, 34, 38, 46], [39,, 41, 45], [56,, 58, 62]]);
            }));

            function emit(_x3, _x4) {
                return _ref.apply(this, arguments);
            }

            return emit;
        }()
    }]);
    return Emitter;
}();

var EEE = function () {
    function EEE() {
        (0, _classCallCheck3.default)(this, EEE);

        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
        }

        this.__eee = new (Function.prototype.bind.apply(Emitter, [null].concat([this], args)))();
    }

    (0, _createClass3.default)(EEE, [{
        key: 'on',
        value: function on() {
            var _eee;

            return (_eee = this.__eee).on.apply(_eee, arguments);
        }
    }, {
        key: 'once',
        value: function once() {
            var _eee2;

            return (_eee2 = this.__eee).once.apply(_eee2, arguments);
        }
    }, {
        key: 'off',
        value: function off() {
            var _eee3;

            return (_eee3 = this.__eee).off.apply(_eee3, arguments);
        }
    }, {
        key: 'emit',
        value: function emit() {
            var _eee4;

            return (_eee4 = this.__eee).emit.apply(_eee4, arguments);
        }
    }]);
    return EEE;
}();

exports.default = EEE;